<?php
/**
 * File: $Id$
 *
 * Authfud User API
 *
 */

/**
 * Login an externally authenticated user
 * @public
 * @param args['uname'] username of user
 * @param args['pass'] dummy password
 * @returns int
 * @return uid on successful login, XARUSER_AUTH_FAILED otherwise
 * @todo rework to make external user info access easily configurable, including LDAP
 */
function authfud_userapi_authenticate_user($args)
{
    extract($args);

    if (!isset($uname) || !isset($pass) || $pass == "") {
        $msg = xarML('Empty uname (#(1)) or pass (not shown).', $uname);
        xarErrorSet(XAR_SYSTEM_EXCEPTION, 'BAD_PARAM',
                       new SystemException(__FILE__.'('.__LINE__.'): '.$msg));
        return XARUSER_AUTH_FAILED;
    }


    // open FUDforum connection
    $connect = authfud__open_fud_connection();
    if (!$connect) return XARUSER_AUTH_FAILED;

    // get user information
    $user_info = authfud__get_fud_userdata($connect,$uname,$pass);
    if (!$user_info) {
        $msg = xarML('Wrong username (#(1)) or pass (not shown).', $uname);
        xarErrorSet(XAR_SYSTEM_EXCEPTION, 'BAD_PARAM',
                       new SystemException(__FILE__.'('.__LINE__.'): '.$msg));
        return XARUSER_AUTH_FAILED;
    }

    // OK, authentication worked
    // now we still have to fetch the $uid for return

    list($dbconn) = xarDBGetConn();
    $xartable = xarDBGetTables();

    // Get user information from roles
    $userRole = xarModAPIFunc('roles',
                              'user',
                              'get',
                              array('uname' => $uname)); 


    if (!$userRole) {
        if (xarModGetVar('authfud','add_user')!='true')
            return XARUSER_AUTH_FAILED;
            
        // add a user that does NOT exist in the database
        // get username from array
        $realname = $user_info['login'];
        // get email from array
        $email = $user_info['email'];

        // call role module to create new user role
        $now = time();
        $rid = xarModAPIFunc('roles',
                             'admin',
                             'create',
                             array('uname' => $uname, 
                                   'realname' => $realname, 
                                   'email' => $email, 
                                   'pass' => $pass,
                                   'date'     => $now,
                                   'valcode'  => 'createdbyfud',
                                   'state'   => 3,
                                   'authmodule'  => 'authfud'));

        if (!$rid)
            return XARUSER_AUTH_FAILED;

        $usergroup = xarModGetVar('authfud','defaultgroup');

        // Get the list of groups
        if (!$groupRoles = xarGetGroups()) return; // throw back

        $groupId = 0;
        while (list($key,$group) = each($groupRoles)) {
            if ($group['name'] == $usergroup) { 
                $groupId = $group['uid'];
                break;
            }
        }

        if ($groupId == 0) return; // throw back

        // Insert the user into the default users group
        if( !xarMakeRoleMemberByID($rid, $groupId))
               return XARUSER_AUTH_FAILED; 
            
    } else {
        $rid = $userRole['uid'];
    }
    
    // Create a session in FUDforum
    authfud__board_login($connect, $uname, $pass);
    
    // close FUDforum connection --may not be necessary
    authfud__close_fud_connection($connect);
    
    return $rid;
}

// PRIVATE FUNCTIONS
function authfud__open_fud_connection()
{
    
    $server = xarModGetVar('authfud','server');
    $uname = xarModGetVar('authfud','username');
    $pwd = xarModGetVar('authfud','password');  
    $database = xarModGetVar('authfud','database');
    $dbtype= xarModGetVar('authfud','dbtype');
    
    $db = NewADOConnection($dbtype);
    if (!$db) {
        $msg = "Fudforum: Loading ADOdb driver '".$server."' has failed: " . $db->ErrorMsg();
        xarErrorSet(XAR_SYSTEM_EXCEPTION, 'NO_CONNECTION',
                       new SystemException(__FILE__.'('.__LINE__.'): '.$msg));
        error_log("FUDforum Error: Loading ADOdb driver '".$server."' failed");
        return;
    }
    $db->Connect($server, $uname, $pwd, $database);

    if (!$db) {
        $msg = "FUdforum: Connection to $server has failed: " . $db->ErrorMsg();
        xarErrorSet(XAR_SYSTEM_EXCEPTION, 'NO_CONNECTION',
                       new SystemException(__FILE__.'('.__LINE__.'): '.$msg));
        error_log("FUDforum Error: Connection to $server failed");
        return;
    }
    return $db;
}

function authfud__board_login($connect,$username,$password)
{
    //Not realized yet.
    return true;
}


function authfud__close_fud_connection($connect)
{
    return true;
}

function authfud__get_fud_userdata($connect,$username,$pass)
{
    $prefix = xarModGetVar('authfud','prefix');
    $password = md5($pass);
    $table = $prefix.'users';

    if($connect)  //just double-checking the connection.
    {
        // connect to the FUDforum database and get the user data
        //$inv_db = mysql_select_db($database, $connect);
        $query = "SELECT * FROM " . $table . " WHERE login=? AND passwd=?";
        
        $connect->SetFetchMode(ADODB_FETCH_ASSOC);
        
        $bindvars = array($username, $password);
        $result =& $connect->Execute($query,$bindvars);
    
        if (!$result) {
            //error
            $msg = "FUDforum: Query to $table has failed: " . $connect->ErrorMsg();
            xarErrorSet(XAR_SYSTEM_EXCEPTION, 'SQL_ERROR',
                new SystemException(__FILE__.'('.__LINE__.'): '.$msg));
            error_log("FUDforum Error: Query to $table failed");
            return false;
        } 
        if ($result->EOF)
        {
            //incorrect login
        $msg = xarML('Wrong username (#(1)) or pass (not shown).', $username);
            xarErrorSet(XAR_SYSTEM_EXCEPTION, 'BAD_PARAM',
                new SystemException(__FILE__.'('.__LINE__.'): '.$msg));
            $result->Close();
            return false;
        }
        else {
            //correct login.  return uid.
                if ($result->fields['users_opt']=='0')
                {
                    //user inactive
                    $msg = xarML('User #(1) not activated.', $username);
                    xarErrorSet(XAR_SYSTEM_EXCEPTION, 'BAD_PARAM',
                        new SystemException(__FILE__.'('.__LINE__.'): '.$msg));
                    $result->Close();
                    return false;    
                }
                else
                {
                    $user_data=$result->fields;
                    $result->Close();
                    return $user_data;
                }
        }
    }
}


?>